Package: FunctionalCalibration
Type: Package
Title: Aggregated Functional Data Calibration using Splines and
        Wavelets
Version: 2.0.0
Authors@R: c(
    person("Vitor", "Perrone", email = "vitor.perrone10@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0009-0009-6923-7712")),
    person("Alex", "Sousa", email = "asousa@unicamp.br", role = c("aut"), comment = c(ORCID = "0000-0001-5887-3638"))
  )
Description: Implements methods for calibrating an aggregated functional data model using 
    wavelets or splines. Each aggregated curve is modeled as a linear combination of 
    component functions and known weights. The component functions are estimated using 
    wavelets or splines. The package is based on dos Santos Sousa (2024) <doi:10.1515/mcma-2023-2016>, 
    Saraiva and Dias (2009) <doi:10.47749/T/UNICAMP.2009.471073> and Johnstone and Silverman (1997) <doi:10.1111/1467-9868.00071>.
URL: https://github.com/VitorRibasP/FunctionalCalibration
Imports: wavethresh
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Depends: R (>= 3.5)
NeedsCompilation: no
Packaged: 2026-08-02 13:20:45 UTC; vitor
Author: Vitor Perrone [aut, cre] (ORCID:
    <https://orcid.org/0009-0009-6923-7712>),
  Alex Sousa [aut] (ORCID: <https://orcid.org/0000-0001-5887-3638>)
Maintainer: Vitor Perrone <vitor.perrone10@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-02 18:40:12 UTC

# Generated by roxygen2: do not edit by hand

export(functional_calibration_splines)
export(functional_calibration_wavelets)
export(plot_aggregated_curve)
export(weight_estimation)
importFrom(grDevices,recordPlot)
importFrom(splines,bs)
importFrom(stats,coef)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,rnorm)
importFrom(wavethresh,threshold)
importFrom(wavethresh,wd)
importFrom(wavethresh,wr)

#' @title Functional Data Calibration with Splines
#'
#' @description
#' This function performs functional calibration based on the following model:
#'
#' \deqn{A_i(x_m) = \displaystyle \sum_{l=1}^{L} y_{il} \alpha_l(x_m) + e_i(x_m), \quad i = 1,...,I, \quad m = 1,...,M = 2^J}
#'
#' where the functions \eqn{\alpha_l(x)} are estimated using spline basis functions.
#'
#' In matrix notation, the model is represented as:
#'
#' \deqn{A = \alpha y + e}
#'
#' @param data A matrix \eqn{M} x \eqn{I} where each column represents one sample of the aggregated function — the matrix \eqn{A} in the model.
#' @param weights A matrix \eqn{L} x \eqn{I} representing the weight values associated with each sample — the matrix \eqn{y} in the model.
#' @param x A numeric vector of values at which the function is evaluated.
#' @param n_functions Number of spline basis functions to be used for estimating \eqn{\alpha_l(x)}.
#'
#' @return
#'
#' The function returns a list containing two objects.
#'
#' \describe{
#'   \item{\code{alpha}}{A matrix with the estimated functional coefficients \eqn{\alpha}.}
#'   \item{\code{Plots}}{A list of plot objects, each representing the corresponding function \eqn{\alpha_l(x)}.}
#'}
#'
#' @references Saraiva, M. A., & Dias, R. (2009). Analise não-parametrica de dados funcionais: uma aplicação a quimiometria (Doctoral dissertation, Master’s thesis, Universidade Estadual de Campinas, Campinas).
#'
#' @examples
#' functional_calibration_splines(simulated_data_spl$data,
#' simulated_data_spl$weights, simulated_data_spl$x)
#' functional_calibration_splines(simulated_data_spl$data,
#' simulated_data_spl$weights, simulated_data_spl$x, 12)
#'
#' @importFrom splines bs
#' @importFrom grDevices recordPlot
#'
#' @export
functional_calibration_splines <- function(data, weights, x,
                                           n_functions = 10) {

  if (ncol(data) != ncol(weights)) {
    stop("Error: data dimension is not compatible with weights dimension.")
  }

  if (inherits(data, "data.frame")) {
    data <- as.matrix(data)
  }

  if (inherits(weights, "data.frame")) {
    weights <- as.matrix(weights)
  }

  X <- as.vector(data)
  d <- matrix(NA, nrow = ncol(data)*length(x), ncol = nrow(weights))
  for (i in 1:nrow(weights)) {
    d[,i] <- rep(weights[i,], each = length(x))
  }

  B <- bs(x, degree = 3, df = n_functions)
  D <- d*do.call(rbind, replicate(ncol(data),
                                  do.call(cbind, replicate(nrow(weights), B[,1], simplify = FALSE))
                                  , simplify = FALSE))

  for (i in 2:n_functions) {
    D <- cbind(D, d*do.call(rbind, replicate(ncol(data),
                                             do.call(cbind, replicate(nrow(weights), B[,i], simplify = FALSE))
                                             , simplify = FALSE)))
  }

  theta <- solve(t(D)%*%D)%*%t(D)%*%X

  alpha <- matrix(NA, nrow = length(x), ncol = nrow(weights))
  for (i in 1:ncol(alpha)) {
    thetas <- theta[seq(i, nrow(theta), by = ncol(alpha))]
    alpha[,i] <- B%*%thetas
  }
  if(is.null(rownames(weights))) {
    titles <- paste0("alpha", 1:nrow(weights))
  } else {
    titles <- rownames(weights)
  }

  Plots <- list()

  for (i in 1:ncol(alpha)) {
    plot(x, alpha[,i], type = "l", lwd = 2,
         main = paste0("Estimated ", titles[i]), xlab = "x", ylab = "y")
    Plots[[i]] <- recordPlot()
  }

  return(list("alpha" = alpha, "Plots" = Plots))
}

#' @title Weight Estimation
#'
#' @description
#' Estimates the weights associated with the functional coefficients
#' \eqn{\alpha_l(x)} using the  using Ordinary Least Squares.
#'
#' The problem can be formulated as:
#'
#' \deqn{A(x) = \displaystyle \sum_{l=1}^{L} y_l \alpha_l(x)}
#'
#' where \eqn{A(x)} is the aggregated function evaluated at each point \eqn{x},
#' \eqn{\alpha_l(x)} are the functional coefficients, and \eqn{y_l} are the weights to be estimated.
#'
#' @param data A numeric vector representing one sample of the aggregated function \eqn{A(x)}, evaluated at a grid of points \eqn{x}.
#' @param alpha A numeric matrix where each column represents the values of a function \eqn{\alpha_l(x)} evaluated at the same grid of points as \code{data}.
#'
#' @return The function returns a vector with the estimated weights obtained using Ordinary Least Squares.
#'
#' @examples
#' weight_estimation(simulated_data_wav$data[,1], simulated_data_wav$alphas)
#'
#' @importFrom stats coef lm
#'
#' @export
weight_estimation <- function(data, alpha) {
  df <- as.data.frame(alpha)
  df$data <- data
  fit <- lm(data ~ . - 1, data = df)
  coefs <- coef(fit)
  return(coefs)
}

#' @title Logistic Density
#'
#' @description
#' Computes the function:
#'
#' \deqn{g(\theta; \tau) = \displaystyle \frac{\exp\left(-\frac{\theta}{\tau}\right)}{\tau \left(1 + \exp\left(-\frac{\theta}{\tau}\right)\right)^2}}
#'
#' @param theta Numeric value of \eqn{\theta}.
#' @param tau Numeric value of \eqn{\tau}.
#'
#' @return A numeric value representing the result of the function \eqn{g(\theta; \tau)} for the specified inputs.
#'
#'@keywords internal
Logistic_Density <- function(theta, tau) {
  return(exp(-theta/tau)/(tau*(1 + exp(-theta/tau))^2))
}
#' @title Bayesian Shrinkage
#'
#' @description
#' A Bayesian shrinkage method applied to empirical coefficients \eqn{d}, aiming to denoise them.
#'
#' The shrinkage function is defined as:
#' \deqn{\delta(d) = \displaystyle \frac{(1 - p) \int_{\mathbb{R}} (\sigma u + d) \, g(\sigma u + d; \tau) \, \phi(u) \, du}{\frac{p}{\sigma} \phi\left( \frac{d}{\sigma} \right) + (1 - p) \int_{\mathbb{R}} g(\sigma u + d; \tau) \, \phi(u) \, du}}
#'
#' where \eqn{\phi(x)} is the probability density function of the standard normal distribution,
#' and \eqn{g(\theta; \tau)} is the logistic density function.
#'
#' @param d Numeric value of the empirical coefficient to be denoised.
#' @param tau Numeric value of \eqn{\tau}.
#' @param p Numeric value of \eqn{p}.
#' @param sigma Numeric value of \eqn{\sigma}.
#' @param MC A logical evaluating to \code{TRUE} or \code{FALSE} indicating if the integrals will be approximated using Monte Carlo.
#'
#' @return A numeric value representing the result of the Bayesian shrinkage applied to the empirical coefficient \eqn{d}.
#'
#' @importFrom stats rnorm dnorm integrate
#'
#' @keywords internal
Bayesian_Shrinkage <- function(d, tau, p, sigma, MC = TRUE) {
  if (MC) {
    N <- rnorm(500)
    return(
      ((1-p)*mean((sigma*N + d) * Logistic_Density(sigma*N + d, tau)))/
        (p/sigma*dnorm(d/sigma) + (1-p)*mean(Logistic_Density(sigma*N + d, tau)))
    )
  } else {
    return(
      ((1-p)*integrate(function(u) (sigma*u + d) * Logistic_Density(sigma*u + d, tau) * dnorm(u), -Inf, Inf)$value)/
        (p/sigma*dnorm(d/sigma) + (1-p)*integrate(function(u) Logistic_Density(sigma*u + d, tau) * dnorm(u), -Inf, Inf)$value)
    )
  }
}
#' @title Universal Shrinkage
#'
#' @description
#' A short description...
#'
#' @param d Numeric value of the empirical coefficient to be denoised
#' @param lambda Numeric value of \eqn{\lambda}
#'
#' @return A numeric value representing the result of the Universal shrinkage applied to the empirical coefficient \eqn{d}.
#'
#' @keywords internal
Universal_Shrinkage <- function(d, lambda) {
  if (abs(d) < lambda) {
    return(0)
  } else {
    return(sign(d)*(abs(d) - lambda))
  }
}
#' @title Functional Data Calibration with Wavelets
#'
#' @description
#' This function performs functional calibration based on the following model:
#'
#' \deqn{A_i(x_m) = \displaystyle \sum_{l=1}^{L} y_{il} \alpha_l(x_m) + e_i(x_m), \quad i = 1,...,I, \quad m = 1,...,M = 2^J}
#'
#' where the functions \eqn{\alpha_l(x)} are estimated using wavelet decomposition.
#'
#' In matrix notation, the model is represented as:
#'
#' \deqn{A = \alpha y + e}
#'
#' @param data A matrix \eqn{M} x \eqn{I} where each column represents one sample of the aggregated function — the matrix \eqn{A} in the model.
#' @param weights A matrix \eqn{L} x \eqn{I} representing the weight values associated with each sample — the matrix \eqn{y} in the model.
#' @param wavelet A string indicating the wavelet family to be used in the Discrete Wavelet Transform (DWT).
#' @param method A string specifying the shrinkage method applied to the empirical wavelet coefficients. Options are: "bayesian", "universal", "sure", "probability", or "cv".
#' @param tau A numeric value for the \eqn{\tau} parameter in the Bayesian shrinkage. If \code{NULL}, it is estimated from the data.
#' @param p A numeric value for the \eqn{p} parameter in the Bayesian shrinkage. If \code{NULL}, it is estimated from the data.
#' @param sigma A numeric value for the \eqn{\sigma} parameter in the Bayesian shrinkage. If \code{NULL}, it is estimated from the data.
#' @param MC A logical evaluating to \code{TRUE} or \code{FALSE} indicating if the integrals in the Bayesian shrinkage are approximated using Monte Carlo simulation.
#' @param type A string indicating whether the thresholding should be "soft" or "hard" (applies only when the method is not "bayesian").
#' @param singular A logical evaluating to \code{TRUE} or \code{FALSE} indicating if it adds a small constant (1e-10) to the diagonal of \eqn{yy^T} to stabilize the matrix inversion.
#' @param corre A logical evaluating to \code{TRUE} or \code{FALSE} indicating if the errors of the data are correlated. Only available for "bayesian" and "universal" methods.
#' @param x A numeric vector of values at which the function is evaluated. If \code{NULL}, the default is the sequence \code{1:nrow(data)}.
#'
#' @return
#'
#' The function returns a list containing two objects:
#'
#' \describe{
#'   \item{\code{alpha}}{A matrix with the estimated functional coefficients \eqn{\alpha}.}
#'   \item{\code{Plots}}{A list of plot objects, each representing the corresponding function \eqn{\alpha_l(x)}.}
#'}
#'
#' @references dos Santos Sousa, A. R. (2024). A wavelet-based method in aggregated functional data analysis. Monte Carlo Methods and Applications, 30(1), 19-30.
#' @references JOHNSTONE, I. M.; SILVERMAN, B. W. Wavelet Threshold Estimators for Data with Correlated Noise. Journal of the Royal Statistical Society: Series B (Statistical Methodology), v. 59, n. 2, p. 319–351, 1997.
#'
#' @examples
#' \donttest{functional_calibration_wavelets(simulated_data_wav$data, simulated_data_wav$weights)}
#' \donttest{functional_calibration_wavelets(simulated_data_wav$data, simulated_data_wav$weights,
#'                                 tau = 5, p = 0.95, sigma = 0.1, x = simulated_data_wav$x)}
#' \donttest{functional_calibration_wavelets(simulated_data_wav$data, simulated_data_wav$weights,
#'                                 method = "universal")}
#' \donttest{functional_calibration_wavelets(simulated_data_cor$data, simulated_data_cor$weights, corre = TRUE)}
#'
#' @importFrom wavethresh wd threshold wr
#' @importFrom stats median
#' @importFrom grDevices recordPlot
#'
#' @export
functional_calibration_wavelets <- function(data, weights, wavelet = "DaubExPhase",
                                            method = "bayesian", tau = 1,
                                            p = NULL, sigma = NULL, MC = TRUE,
                                            type = "soft", singular = FALSE,
                                            corre = FALSE, x = NULL) {

  if (ncol(data) != ncol(weights)) {
    stop("Error: data dimension is not compatible with weights dimension.")
  }

  if (bitwAnd(nrow(data), nrow(data) - 1) != 0) {
    stop("Error: dataset length is not a power of 2.")
  }

  if (!(method %in% c("bayesian", "universal", "sure", "probability", "cv"))) {
    stop("Error: the methods of shrinkage avaiable are: bayesian, universal, sure, probability and cv.")
  }

  if(corre & !(method %in% c("bayesian", "universal"))) {
    stop("Error: only the bayesian and universal shrinkages are avaiable for correlated errors.")
  }

  if (singular) {
    e <- 1e-10
  } else {
    e <- 0
  }

  if (inherits(data, "data.frame")) {
    data <- as.matrix(data)
  }

  if (inherits(weights, "data.frame")) {
    weights <- as.matrix(weights)
  }

  if (method == "bayesian") {

    D <- matrix(NA, nrow = nrow(data), ncol = ncol(data))

    for (i in 1:ncol(data)) {
      DWT <- wd(data[,i], family = wavelet)
      D[,i] <- c(DWT$D, DWT$C[length(DWT$C)])
    }

    if (is.null(sigma)) {
      if (corre) {
        sigma <- matrix(NA, nrow = log2(nrow(data)):1, ncol = ncol(data))
        ns <- 2^(log2(nrow(data)):1 - 1)
        n_sum <- c(0, cumsum(ns))
        for(i in 1:ncol(data)) {
          for(j in 1:length(ns)) {
            sigma[j, i] <- median(abs(D[,i][(n_sum[j]+1):n_sum[j+1]]))
          }
        }
        lambda <- sqrt(2*log(nrow(data)))*sigma
      } else {
        sigma <- numeric(ncol(data))
        for (i in 1:ncol(data)) {
          sigma[i] <- median(abs(D[,i][1:(2^(log2(nrow(data)) - 1))]))/0.6745
        }
        sigma <- matrix(sigma, nrow = log2(nrow(data)):1, ncol = ncol(data),
                        byrow = TRUE)
      }
    } else {
      sigma <- matrix(sigma, nrow = log2(nrow(data)):1, ncol = ncol(data))
    }

    if (is.null(p)) {
      p <- 1 - 1/((log2(nrow(data)) - 1):0 + 1)^2
    } else {
      p <- rep(p, log2(nrow(data)))
    }
    delta_D <- matrix(NA, nrow = nrow(data), ncol = ncol(data))
    for (i in 1:ncol(data)) {
      k <- 1
      for (j in 1:log2(nrow(data))) {
        for (l in 1:2^(log2(nrow(data)) - j)) {
          delta_D[k, i] <- Bayesian_Shrinkage(D[k, i], tau, p[j], sigma[j, i], MC)
          k <- k + 1 }
      }
      delta_D[k, i] <- Bayesian_Shrinkage(D[k, i], tau, p[length(p)], sigma[j, i], MC)
    }

  } else {
    if (corre) {
      sigma <- matrix(NA, nrow = log2(nrow(data)):1, ncol = ncol(data))
      ns <- 2^(log2(nrow(data)):1 - 1)
      n_sum <- c(0, cumsum(ns))
      for(i in 1:ncol(data)) {
        for(j in 1:length(ns)) {
          sigma[j, i] <- median(abs(D[,i][(n_sum[j]+1):n_sum[j+1]]))
        }
      }
      lambda <- sqrt(2*log(nrow(data)))*sigma

      delta_D <- matrix(NA, nrow = nrow(data), ncol = ncol(data))
      for (i in 1:ncol(data)) {
        k <- 1
        for (j in 1:log2(nrow(data))) {
          for (l in 1:2^(log2(nrow(data)) - j)) {
            delta_D[k, i] <- Universal_Shrinkage(D[k, i], lambda[j, i])
            k <- k + 1 }
        }
        delta_D[k, i] <- Universal_Shrinkage(D[k, i], lambda[length(p), i])
      }

    } else {
      delta_D <- matrix(NA, nrow = nrow(data), ncol = ncol(data))
      for (i in 1:ncol(data)) {
        DWT <- wd(data[,i], family = wavelet)
        DWT <- threshold(DWT, policy = method, type = type)
        delta_D[,i] <- c(DWT$D, DWT$C[length(DWT$C)])
      }
    }
  }
  Gama <- delta_D%*%t(weights)%*%solve(weights%*%t(weights) + e*diag(nrow(weights)))

  alpha <- matrix(NA, nrow = nrow(Gama), ncol = ncol(Gama))
  for (i in 1:ncol(Gama)) {
    DWT$D <- Gama[-nrow(Gama),i]
    DWT$C[length(DWT$C)] <- Gama[nrow(Gama),i]
    alpha[,i] <- wr(DWT)
  }

  if (is.null(x)) {
    x <- 1:nrow(alpha)
  }

  if(is.null(rownames(weights))) {
    titles <- paste0("alpha", 1:nrow(weights))
  } else {
    titles <- rownames(weights)
  }

  Plots <- list()

  for (i in 1:ncol(alpha)) {
    plot(x, alpha[,i], type = "l", lwd = 2,
         main = paste0("Estimated ", titles[i]), xlab = "x", ylab = "y")
    Plots[[i]] <- recordPlot()
  }

  return(list("alpha" = alpha, "Plots" = Plots))
}

#' @title Simulated Data Correlated
#' @description
#' This is a simulated dataset designed to illustrate the functionalities of the package.
#' It contains 100 samples of aggregated data generated from two functions, \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)}, with added noise AR(1) process with autoregressive parameter 0.5 and Gaussian process noise distributed as \eqn{N(0, 0.0075)}.
#'
#' The functions used in the simulation are:
#'
#' \deqn{ \alpha_1(x) = \sum_{j=1}^{11} h_j \left(1 + \left|\frac{x - x_j}{w_j}\right|\right)^{-4} \quad \alpha_2(x) = \sqrt{x(1-x)}\sin\left(\frac{2\pi(1 + 0.05)}{x + 0.05}\right)}
#'
#' where
#' \eqn{x = (0.1,\,0.13,\,0.15,\,0.23,\,0.25,\,0.40,\,0.44,\,0.65,\,0.76,\,0.78,\,0.81)},
#'
#' \eqn{h = (4,\,5,\,3,\,4,\,5,\,4.2,\,2.1,\,4.3,\,3.1,\,5.1,\,4.2)},
#'
#' \eqn{w = (0.005,\,0.005,\,0.006,\,0.01,\,0.01,\,0.03,\,0.01,\,0.01,\,0.005,\,0.008,\,0.005)}.
#'
#'
#' The simulations were performed over an equally spaced grid of 1024 points in the interval [0, 1].
#' These functions were linearly combined using random concentrations to generate the samples, with the addition of AR(1) noise.
#'
#' @return
#'
#' \describe{
#'   \item{\code{data}}{A data frame with 1024 rows and 100 columns. \cr
#'                          Each column represents one sample of the aggregated functions with AR(1) noise.}
#'   \item{\code{weigths}}{A data frame with 2 rows and 100 columns. \cr
#'                                Each column contains the random concentrations used to aggregate the two functions in each sample.}
#'   \item{\code{x}}{A numeric vector of length 1024. \cr
#'                   The grid of x-values used in the simulation, equally spaced from 0 to 1.}
#'   \item{\code{alphas}}{A data frame with 1024 rows and 2 columns. \cr
#'                            The true values of the functions \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)} evaluated over the x grid.}
#' }
"simulated_data_cor"
#' @title Simulated Data Splines
#' @description
#' This is a simulated dataset designed to illustrate the functionalities of the package.
#' It contains 100 samples of aggregated data generated from two functions, \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)}, with added Gaussian noise \eqn{N(0, 0.01)}.
#'
#' The functions used in the simulation are:
#'
#' \deqn{\alpha_1(x) = \sin(5x) e^{-x^2} \quad \alpha_2(x) = \frac{1.2 \log(1 + 9x)}{\log(10)} e^{-x}}
#'
#' The simulations were performed over an equally spaced grid of 1024 points in the interval [0, 1].
#' These functions were linearly combined using random concentrations to generate the samples, with the addition of Gaussian noise.
#'
#' @return
#'
#' \describe{
#'   \item{\code{data}}{A data frame with 1024 rows and 100 columns. \cr
#'                          Each column represents one sample of the aggregated functions with Gaussian noise.}
#'   \item{\code{weigths}}{A data frame with 2 rows and 100 columns. \cr
#'                                Each column contains the random concentrations used to aggregate the two functions in each sample.}
#'   \item{\code{x}}{A numeric vector of length 1024. \cr
#'                   The grid of x-values used in the simulation, equally spaced from 0 to 1.}
#'   \item{\code{alphas}}{A data frame with 1024 rows and 2 columns. \cr
#'                            The true values of the functions \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)} evaluated over the x grid.}
#' }
"simulated_data_spl"
#' @title Simulated Data Wavelets
#' @description
#' This is a simulated dataset designed to illustrate the functionalities of the package.
#' It contains 100 samples of aggregated data generated from two functions, \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)}, with added Gaussian noise \eqn{N(0, 0.01)}.
#'
#' The functions used in the simulation are:
#'
#' \deqn{ \alpha_1(x) = \sum_{j=1}^{11} h_j \left(1 + \left|\frac{x - x_j}{w_j}\right|\right)^{-4} \quad \alpha_2(x) = \sqrt{x(1-x)}\sin\left(\frac{2\pi(1 + 0.05)}{x + 0.05}\right)}
#'
#' where
#' \eqn{x = (0.1,\,0.13,\,0.15,\,0.23,\,0.25,\,0.40,\,0.44,\,0.65,\,0.76,\,0.78,\,0.81)},
#'
#' \eqn{h = (4,\,5,\,3,\,4,\,5,\,4.2,\,2.1,\,4.3,\,3.1,\,5.1,\,4.2)},
#'
#' \eqn{w = (0.005,\,0.005,\,0.006,\,0.01,\,0.01,\,0.03,\,0.01,\,0.01,\,0.005,\,0.008,\,0.005)}.
#'
#' The simulations were performed over an equally spaced grid of 1024 points in the interval [0, 1].
#' These functions were linearly combined using random concentrations to generate the samples, with the addition of Gaussian noise.
#'
#' @return
#'
#' \describe{
#'   \item{\code{data}}{A data frame with 1024 rows and 100 columns. \cr
#'                          Each column represents one sample of the aggregated functions with Gaussian noise.}
#'   \item{\code{weigths}}{A data frame with 2 rows and 100 columns. \cr
#'                                Each column contains the random concentrations used to aggregate the two functions in each sample.}
#'   \item{\code{x}}{A numeric vector of length 1024. \cr
#'                   The grid of x-values used in the simulation, equally spaced from 0 to 1.}
#'   \item{\code{alphas}}{A data frame with 1024 rows and 2 columns. \cr
#'                            The true values of the functions \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)} evaluated over the x grid.}
#' }
"simulated_data_wav"

#' @title Aggregated Curve Plot
#'
#' @description
#'
#' Generates the plot of the aggregated curve based on the functional coefficients and their corresponding weights.
#' The aggregated curve is computed as:
#'
#' \deqn{A(x) = \displaystyle \sum_{l=1}^{L} y_l \alpha_l(x)}
#'
#' @param alpha A numeric matrix where each column represents the values of a function \eqn{\alpha_l(x)} evaluated at each point in \eqn{x}.
#' @param weights A numeric vector with the weight values corresponding to each function \eqn{\alpha_l(x)}.

#' @param title A string specifying the title of the plot.
#' @param x A numeric vector of values at which the function is evaluated. If \code{NULL}, the default is the sequence \code{1:nrow(alpha)}.
#'
#' @return The function returns the plot of the aggregated function.
#'
#' @examples
#' plot_aggregated_curve(simulated_data_wav$alphas, c(0.7, 0.3))
#' plot_aggregated_curve(simulated_data_wav$alphas, c(0.7, 0.3),
#'                       "Aggregated Curve Example", simulated_data_wav$x)
#'
#' @importFrom grDevices recordPlot
#'
#' @export
plot_aggregated_curve <- function(alpha, weights, title = NULL, x = NULL) {
  if (is.null(x)) {
    x <- 1:nrow(alpha)
  }
  if (inherits(alpha, "data.frame")) {
    alpha <- as.matrix(alpha)
  }
  plot(x, alpha%*%weights, type = "l", xlab = "x", ylab = "y", main = title, lwd = 2)
  Plot <- recordPlot()
  return(Plot)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_aggregated_curve.R
\name{plot_aggregated_curve}
\alias{plot_aggregated_curve}
\title{Aggregated Curve Plot}
\usage{
plot_aggregated_curve(alpha, weights, title = NULL, x = NULL)
}
\arguments{
\item{alpha}{A numeric matrix where each column represents the values of a function \eqn{\alpha_l(x)} evaluated at each point in \eqn{x}.}

\item{weights}{A numeric vector with the weight values corresponding to each function \eqn{\alpha_l(x)}.}

\item{title}{A string specifying the title of the plot.}

\item{x}{A numeric vector of values at which the function is evaluated. If \code{NULL}, the default is the sequence \code{1:nrow(alpha)}.}
}
\value{
The function returns the plot of the aggregated function.
}
\description{
Generates the plot of the aggregated curve based on the functional coefficients and their corresponding weights.
The aggregated curve is computed as:

\deqn{A(x) = \displaystyle \sum_{l=1}^{L} y_l \alpha_l(x)}
}
\examples{
plot_aggregated_curve(simulated_data_wav$alphas, c(0.7, 0.3))
plot_aggregated_curve(simulated_data_wav$alphas, c(0.7, 0.3),
                      "Aggregated Curve Example", simulated_data_wav$x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_calibration_wavelets.R
\name{Logistic_Density}
\alias{Logistic_Density}
\title{Logistic Density}
\usage{
Logistic_Density(theta, tau)
}
\arguments{
\item{theta}{Numeric value of \eqn{\theta}.}

\item{tau}{Numeric value of \eqn{\tau}.}
}
\value{
A numeric value representing the result of the function \eqn{g(\theta; \tau)} for the specified inputs.
}
\description{
Computes the function:

\deqn{g(\theta; \tau) = \displaystyle \frac{\exp\left(-\frac{\theta}{\tau}\right)}{\tau \left(1 + \exp\left(-\frac{\theta}{\tau}\right)\right)^2}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_calibration_wavelets.R
\name{Bayesian_Shrinkage}
\alias{Bayesian_Shrinkage}
\title{Bayesian Shrinkage}
\usage{
Bayesian_Shrinkage(d, tau, p, sigma, MC = TRUE)
}
\arguments{
\item{d}{Numeric value of the empirical coefficient to be denoised.}

\item{tau}{Numeric value of \eqn{\tau}.}

\item{p}{Numeric value of \eqn{p}.}

\item{sigma}{Numeric value of \eqn{\sigma}.}

\item{MC}{A logical evaluating to \code{TRUE} or \code{FALSE} indicating if the integrals will be approximated using Monte Carlo.}
}
\value{
A numeric value representing the result of the Bayesian shrinkage applied to the empirical coefficient \eqn{d}.
}
\description{
A Bayesian shrinkage method applied to empirical coefficients \eqn{d}, aiming to denoise them.

The shrinkage function is defined as:
\deqn{\delta(d) = \displaystyle \frac{(1 - p) \int_{\mathbb{R}} (\sigma u + d) \, g(\sigma u + d; \tau) \, \phi(u) \, du}{\frac{p}{\sigma} \phi\left( \frac{d}{\sigma} \right) + (1 - p) \int_{\mathbb{R}} g(\sigma u + d; \tau) \, \phi(u) \, du}}

where \eqn{\phi(x)} is the probability density function of the standard normal distribution,
and \eqn{g(\theta; \tau)} is the logistic density function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_calibration_wavelets.R
\name{functional_calibration_wavelets}
\alias{functional_calibration_wavelets}
\title{Functional Data Calibration with Wavelets}
\usage{
functional_calibration_wavelets(
  data,
  weights,
  wavelet = "DaubExPhase",
  method = "bayesian",
  tau = 1,
  p = NULL,
  sigma = NULL,
  MC = TRUE,
  type = "soft",
  singular = FALSE,
  corre = FALSE,
  x = NULL
)
}
\arguments{
\item{data}{A matrix \eqn{M} x \eqn{I} where each column represents one sample of the aggregated function — the matrix \eqn{A} in the model.}

\item{weights}{A matrix \eqn{L} x \eqn{I} representing the weight values associated with each sample — the matrix \eqn{y} in the model.}

\item{wavelet}{A string indicating the wavelet family to be used in the Discrete Wavelet Transform (DWT).}

\item{method}{A string specifying the shrinkage method applied to the empirical wavelet coefficients. Options are: "bayesian", "universal", "sure", "probability", or "cv".}

\item{tau}{A numeric value for the \eqn{\tau} parameter in the Bayesian shrinkage. If \code{NULL}, it is estimated from the data.}

\item{p}{A numeric value for the \eqn{p} parameter in the Bayesian shrinkage. If \code{NULL}, it is estimated from the data.}

\item{sigma}{A numeric value for the \eqn{\sigma} parameter in the Bayesian shrinkage. If \code{NULL}, it is estimated from the data.}

\item{MC}{A logical evaluating to \code{TRUE} or \code{FALSE} indicating if the integrals in the Bayesian shrinkage are approximated using Monte Carlo simulation.}

\item{type}{A string indicating whether the thresholding should be "soft" or "hard" (applies only when the method is not "bayesian").}

\item{singular}{A logical evaluating to \code{TRUE} or \code{FALSE} indicating if it adds a small constant (1e-10) to the diagonal of \eqn{yy^T} to stabilize the matrix inversion.}

\item{corre}{A logical evaluating to \code{TRUE} or \code{FALSE} indicating if the errors of the data are correlated. Only available for "bayesian" and "universal" methods.}

\item{x}{A numeric vector of values at which the function is evaluated. If \code{NULL}, the default is the sequence \code{1:nrow(data)}.}
}
\value{
The function returns a list containing two objects:

\describe{
  \item{\code{alpha}}{A matrix with the estimated functional coefficients \eqn{\alpha}.}
  \item{\code{Plots}}{A list of plot objects, each representing the corresponding function \eqn{\alpha_l(x)}.}
}
}
\description{
This function performs functional calibration based on the following model:

\deqn{A_i(x_m) = \displaystyle \sum_{l=1}^{L} y_{il} \alpha_l(x_m) + e_i(x_m), \quad i = 1,...,I, \quad m = 1,...,M = 2^J}

where the functions \eqn{\alpha_l(x)} are estimated using wavelet decomposition.

In matrix notation, the model is represented as:

\deqn{A = \alpha y + e}
}
\examples{
\donttest{functional_calibration_wavelets(simulated_data_wav$data, simulated_data_wav$weights)}
\donttest{functional_calibration_wavelets(simulated_data_wav$data, simulated_data_wav$weights,
                                tau = 5, p = 0.95, sigma = 0.1, x = simulated_data_wav$x)}
\donttest{functional_calibration_wavelets(simulated_data_wav$data, simulated_data_wav$weights,
                                method = "universal")}
\donttest{functional_calibration_wavelets(simulated_data_cor$data, simulated_data_cor$weights, corre = TRUE)}

}
\references{
dos Santos Sousa, A. R. (2024). A wavelet-based method in aggregated functional data analysis. Monte Carlo Methods and Applications, 30(1), 19-30.

JOHNSTONE, I. M.; SILVERMAN, B. W. Wavelet Threshold Estimators for Data with Correlated Noise. Journal of the Royal Statistical Society: Series B (Statistical Methodology), v. 59, n. 2, p. 319–351, 1997.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_calibration_wavelets.R
\name{Universal_Shrinkage}
\alias{Universal_Shrinkage}
\title{Universal Shrinkage}
\usage{
Universal_Shrinkage(d, lambda)
}
\arguments{
\item{d}{Numeric value of the empirical coefficient to be denoised}

\item{lambda}{Numeric value of \eqn{\lambda}}
}
\value{
A numeric value representing the result of the Universal shrinkage applied to the empirical coefficient \eqn{d}.
}
\description{
A short description...
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated_data_spl}
\alias{simulated_data_spl}
\title{Simulated Data Splines}
\format{
An object of class \code{list} of length 4.
}
\usage{
simulated_data_spl
}
\value{
\describe{
  \item{\code{data}}{A data frame with 1024 rows and 100 columns. \cr
                         Each column represents one sample of the aggregated functions with Gaussian noise.}
  \item{\code{weigths}}{A data frame with 2 rows and 100 columns. \cr
                               Each column contains the random concentrations used to aggregate the two functions in each sample.}
  \item{\code{x}}{A numeric vector of length 1024. \cr
                  The grid of x-values used in the simulation, equally spaced from 0 to 1.}
  \item{\code{alphas}}{A data frame with 1024 rows and 2 columns. \cr
                           The true values of the functions \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)} evaluated over the x grid.}
}
}
\description{
This is a simulated dataset designed to illustrate the functionalities of the package.
It contains 100 samples of aggregated data generated from two functions, \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)}, with added Gaussian noise \eqn{N(0, 0.01)}.

The functions used in the simulation are:

\deqn{\alpha_1(x) = \sin(5x) e^{-x^2} \quad \alpha_2(x) = \frac{1.2 \log(1 + 9x)}{\log(10)} e^{-x}}

The simulations were performed over an equally spaced grid of 1024 points in the interval [0, 1].
These functions were linearly combined using random concentrations to generate the samples, with the addition of Gaussian noise.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated_data_cor}
\alias{simulated_data_cor}
\title{Simulated Data Correlated}
\format{
An object of class \code{list} of length 4.
}
\usage{
simulated_data_cor
}
\value{
\describe{
  \item{\code{data}}{A data frame with 1024 rows and 100 columns. \cr
                         Each column represents one sample of the aggregated functions with AR(1) noise.}
  \item{\code{weigths}}{A data frame with 2 rows and 100 columns. \cr
                               Each column contains the random concentrations used to aggregate the two functions in each sample.}
  \item{\code{x}}{A numeric vector of length 1024. \cr
                  The grid of x-values used in the simulation, equally spaced from 0 to 1.}
  \item{\code{alphas}}{A data frame with 1024 rows and 2 columns. \cr
                           The true values of the functions \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)} evaluated over the x grid.}
}
}
\description{
This is a simulated dataset designed to illustrate the functionalities of the package.
It contains 100 samples of aggregated data generated from two functions, \eqn{\alpha_1(x)} and \eqn{\alpha_2(x)}, with added noise AR(1) process with autoregressive parameter 0.5 and Gaussian process noise distributed as \eqn{N(0, 0.0075)}.

The functions used in the simulation are:

\deqn{ \alpha_1(x) = \sum_{j=1}^{11} h_j \left(1 + \left|\frac{x - x_j}{w_j}\right|\right)^{-4} \quad \alpha_2(x) = \sqrt{x(1-x)}\sin\left(\frac{2\pi(1 + 0.05)}{x + 0.05}\right)}

where
\eqn{x = (0.1,\,0.13,\,0.15,\,0.23,\,0.25,\,0.40,\,0.44,\,0.65,\,0.76,\,0.78,\,0.81)},

\eqn{h = (4,\,5,\,3,\,4,\,5,\,4.2,\,2.1,\,4.3,\,3.1,\,5.1,\,4.2)},

\eqn{w = (0.005,\,0.005,\,0.006,\,0.01,\,0.01,\,0.03,\,0.01,\,0.01,\,0.005,\,0.008,\,0.005)}.


The simulations were performed over an equally spaced grid of 1024 points in the interval [0, 1].
These functions were linearly combined using random concentrations to generate the samples, with the addition of AR(1) noise.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_calibration_splines.R
\name{functional_calibration_splines}
\alias{functional_calibration_splines}
\title{Functional Data Calibration with Splines}
\usage{
functional_calibration_splines(data, weights, x, n_functions = 10)
}
\arguments{
\item{data}{A matrix \eqn{M} x \eqn{I} where each column represents one sample of the aggregated function — the matrix \eqn{A} in the model.}

\item{weights}{A matrix \eqn{L} x \eqn{I} representing the weight values associated with each sample — the matrix \eqn{y} in the model.}

\item{x}{A numeric vector of values at which the function is evaluated.}

\item{n_functions}{Number of spline basis functions to be used for estimating \eqn{\alpha_l(x)}.}
}
\value{
The function returns a list containing two objects.

\describe{
  \item{\code{alpha}}{A matrix with the estimated functional coefficients \eqn{\alpha}.}
  \item{\code{Plots}}{A list of plot objects, each representing the corresponding function \eqn{\alpha_l(x)}.}
}
}
\description{
This function performs functional calibration based on the following model:

\deqn{A_i(x_m) = \displaystyle \sum_{l=1}^{L} y_{il} \alpha_l(x_m) + e_i(x_m), \quad i = 1,...,I, \quad m = 1,...,M = 2^J}

where the functions \eqn{\alpha_l(x)} are estimated using spline basis functions.

In matrix notation, the model is represented as:

\deqn{A = \alpha y + e}
}
\examples{
functional_calibration_splines(simulated_data_spl$data,
simulated_data_spl$weights, simulated_data_spl$x)
functional_calibration_splines(simulated_data_spl$data,
simulated_data_spl$weights, simulated_data_spl$x, 12)

}
\references{
Saraiva, M. A., & Dias, R. (2009). Analise não-parametrica de dados funcionais: uma aplicação a quimiometria (Doctoral dissertation, Master’s thesis, Universidade Estadual de Campinas, Campinas).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_aggregated_data_estimation.R
\name{weight_estimation}
\alias{weight_estimation}
\title{Weight Estimation}
\usage{
weight_estimation(data, alpha)
}
\arguments{
\item{data}{A numeric vector representing one sample of the aggregated function \eqn{A(x)}, evaluated at a grid of points \eqn{x}.}

\item{alpha}{A numeric matrix where each column represents the values of a function \eqn{\alpha_l(x)} evaluated at the same grid of points as \code{data}.}
}
\value{
The function returns a vector with the estimated weights obtained using Ordinary Least Squares.
}
\description{
Estimates the weights associated with the functional coefficients
\eqn{\alpha_l(x)} using the  using Ordinary Least Squares.

The problem can be formulated as:

\deqn{A(x) = \displaystyle \sum_{l=1}^{L} y_l \alpha_l(x)}

where \eqn{A(x)} is the aggregated function evaluated at each point \eqn{x},
\eqn{\alpha_l(x)} are the functional coefficients, and \eqn{y_l} are the weights to be estimated.
}
\examples{
weight_estimation(simulated_data_wav$data[,1], simulated_data_wav$alphas)

}
